#! /usr/bin/env node

const Promise = require('bluebird');
global.Promise = Promise;

const pkg = require('./package.json');
const program = require('commander');
const chalk = require('chalk');
const { logVersionCheck } = require('validate-package-version');
const F = require('lodash/fp');
const fs = require('fs');
const { promisify } = require('util');
const isEmail = require('./lib/is-email');
const exec = require('./lib/exec');
const ora = require('./lib/ora');
const { prompt, requireValue } = require('./lib/prompt');
const template = require('./lib/nginx-template');

const stat = promisify(fs.stat);
const writeFileAsync = promisify(fs.writeFile);

const NGINX_DEFAULT_CONFIG_PATH = `/etc/nginx/sites-available/default`;

const tryCatchCmd = (func) => async (...args) => {
    try {
        const success = await func(...args);
        return process.exit(success ? 0 : 1);
    } catch (error) {
        console.log(chalk.red(error.message));
        console.error(error.stack);
        return process.exit(1);
    }
};

const act = (func, ignoreVersionCheck = false) => async (...args) => {
    if (!ignoreVersionCheck) {
        await logVersionCheck(pkg);
    }
    return tryCatchCmd(func)(...args);
};

const collectOption = (value, coll) => {
    const newColl = F.isNil(coll) ? [] : coll;
    newColl.push(value);
    return newColl;
};

const questions = ({ email, domainNames }) => [
    {
        type: 'input',
        name: 'email',
        message: 'What email would you like to use for letsencrypt?',
        default: email,
        validate: (value) => (isEmail(value) ? true : `You must provide a valid email!`),
        filter: F.trim,
    },
    {
        type: 'input',
        name: 'domainNames',
        message: 'What domain name(s) are you using (separate by commas)?',
        default: domainNames,
        validate: requireValue('domain name'),
        filter: F.compose(
            F.map(F.trim),
            F.split(',')
        ),
    },
];

const commands = ({ email, domainNames }) => [
    `apt-get update -y`,
    `apt-get install nginx -y`,
    `ufw allow 2376`,
    `ufw allow ssh`,
    `ufw allow 'Nginx Full'`,
    `ufw --force enable`,
    `add-apt-repository ppa:certbot/certbot -y`,
    `apt-get update -y`,
    `apt-get install python-certbot-nginx -y`,
    `certbot --nginx --agree-tos -n -m ${email} ${domainNames.map((x) => `-d ${x}`).join(' ')}`,
    `systemctl reload nginx`,
    `certbot renew --dry-run`,
];

const execCmd = async (cmd, desc) => {
    ora.start(desc || cmd);
    await exec(cmd);
    ora.stop();
    console.log(chalk.green('✔'), desc || cmd);
};

program
    .command('init')
    .description('Easy server setup (WARNING: does mess with filesystem / existing nginx setup)')
    .option('-e, --email <email>', 'Letsencrypt email address')
    .option('-d, --domain <domain>', 'Domain name', collectOption)
    .option(
        '-p, --proxy <proxy>',
        'Domain that is used for nginx proxy_pass (Default is http://localhost:5000)'
    )
    .option('-n, --non-interactive', 'Non interactive mode')
    .action(
        act(async ({ email: e, domain: d, nonInteractive, proxy = 'http://localhost:5000' }) => {
            const qs = questions({ email: e, domainNames: d });
            const { email, domainNames } = await prompt(qs, { skipDefaultQuestions: true });
            const { primaryDomain } =
                domainNames.length === 1 || nonInteractive
                    ? { primaryDomain: F.head(domainNames) }
                    : await prompt([
                          {
                              type: 'list',
                              choices: domainNames,
                              name: 'primaryDomain',
                              message:
                                  'What domain would you like all your traffic is redirected to (primary domain)?',
                          },
                      ]);

            ora.start(`Updating ${chalk.underline(NGINX_DEFAULT_CONFIG_PATH)} with bare config`);
            await writeFileAsync(NGINX_DEFAULT_CONFIG_PATH, template.defaultServer, 'utf8');
            ora.stop();
            console.log(
                chalk.green('✔'),
                `Updated ${chalk.underline(NGINX_DEFAULT_CONFIG_PATH)} with bare config`
            );

            const cmds = commands({ email, domainNames });
            await Promise.each(cmds, async (cmd) => {
                await execCmd(cmd);
                return true;
            });

            let letsencryptDirPath = null;
            ora.start('Finding letsencrypt dir name');
            for (let i = 0; i < domainNames.length; i++) {
                try {
                    const domain = domainNames[i];
                    const dirpath = `/etc/letsencrypt/live/${domain}`;
                    await stat(`${dirpath}/fullchain.pem`);
                    letsencryptDirPath = dirpath;
                    break;
                } catch (error) {}
            }
            ora.stop();
            if (F.isEmpty(letsencryptDirPath)) {
                throw new Error(
                    'Couldnt find lets encrypt directory (looking for fullchain.pem file)'
                );
            }
            console.log(
                chalk.green('✔'),
                `Using letsencrypt directory ${chalk.underline(letsencryptDirPath)}`
            );

            const nginxStr = template.make({
                letsencryptDirPath,
                primaryDomain,
                alternativeDomainNames: domainNames.filter((x) => x !== primaryDomain),
                proxyPass: proxy,
            });

            ora.start(`Updating ${chalk.underline(NGINX_DEFAULT_CONFIG_PATH)}`);
            await writeFileAsync(NGINX_DEFAULT_CONFIG_PATH, nginxStr, 'utf8');
            ora.stop();
            console.log(chalk.green('✔'), `Updated ${chalk.underline(NGINX_DEFAULT_CONFIG_PATH)}`);

            await execCmd(`systemctl reload nginx`);
            await execCmd(`certbot renew --dry-run`);

            console.log('🍻 ', 'All done!');
            return true;
        })
    );

program
    .version(pkg.version)
    .description(pkg.description)
    .parse(process.argv);

if (program.args.length === 0) {
    console.log(chalk.red('Please specify a command. Use --help to learn more.'));
    process.exit(1);
}
